/*
 ## Cypress USB 3.0 Platform source file (cyfxgpioapp.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2018,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
*/

/* This file implements a simple GPIO application example. */

/* This example illustrates the use of the FX3 firmware APIs to implement
 * a simple GPIO application example.
 *
 * The example illustrates the usage of simple GPIO API to set and get 
 * the status of the pin and the usage of GPIO interrupts.
 *
 * The example uses GPIO 21 as output. It toggles this pin ON and OFF 
 * at an interval of 2s.
 *
 * GPIO 45 is used as the input GPIO. Interrupts are enabled and a 
 * callback is registered for the GPIO edge interrupts both positive 
 * and negative edges.
 *
 * GPIO 21 is located at pin 2 of the jumper J100 on the DVK board.
 * GPIO 45 is located on the DVK board at the test point TP13.
 */
//CyU3PRegisterGpioCallBack
//CyU3PGpioSimpleSetValue
#include <cyu3system.h>
#include <cyu3os.h>
#include <cyu3error.h>
#include <cyu3gpio.h>
#include <cyu3uart.h>
#include"i2c.h"
//#include"i2c1.c"
//#include"i2c.h"

#include <stdint.h>
#include <stdlib.h>
#define I2C_DATA 59
#define I2C_CLK  58

int32_t val1=0;
#define HIGH 1
#define LOW  0

#define CY_FX_GPIOAPP_THREAD_STACK       (0x0400)   /* GPIO application thread stack size */
#define CY_FX_GPIOAPP_THREAD_PRIORITY    (8)        /* GPIO application thread priority */

#define CY_FX_GPIOAPP_GPIO_HIGH_EVENT    (1 << 0)   /* GPIO high event */
#define CY_FX_GPIOAPP_GPIO_LOW_EVENT     (1 << 1)   /* GPIO low event */

CyU3PThread gpioOutputThread;   /* GPIO thread structure */
CyU3PThread gpioInputThread;    /* GPIO thread structure */
CyU3PEvent glFxGpioAppEvent;    /* GPIO input event group. */

CyBool_t highlow;
CyBool_t highlow1;
CyBool_t highlow2;
uvint32_t *gpio_50_reg      = (uvint32_t*) 0xE00011C8;
uvint32_t *gpio_58_reg      = (uvint32_t*) 0xE00011E8;
uvint32_t *gpio_59_reg      = (uvint32_t*) 0xE00011EC;

//CyU3PGpioIntrCb_t CyFxGpioIntrCb1;
/* Application error handler. */
void
CyFxAppErrorHandler (
        CyU3PReturnStatus_t apiRetStatus    /* API return status */
        )
{
    /* Application failed with the error code apiRetStatus */

    /* Add custom debug or recovery actions here */

    /* Loop indefinitely */
    for (;;)
    {
        /* Thread sleep : 100 ms */
        CyU3PThreadSleep (100);
    }
}

/* Initialize the debug module with UART. */
CyU3PReturnStatus_t
CyFxDebugInit (
        void)
{
    CyU3PUartConfig_t uartConfig;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    /* Initialize and configure the UART for logging. */
    status = CyU3PUartInit ();
    if (status != CY_U3P_SUCCESS)
    {
        return status;
    }

    CyU3PMemSet ((uint8_t *)&uartConfig, 0, sizeof (uartConfig));
    uartConfig.baudRate = CY_U3P_UART_BAUDRATE_115200;
    uartConfig.stopBit  = CY_U3P_UART_ONE_STOP_BIT;
    uartConfig.parity   = CY_U3P_UART_NO_PARITY;
    uartConfig.txEnable = CyTrue;
    uartConfig.rxEnable = CyFalse;
    uartConfig.flowCtrl = CyFalse;
    uartConfig.isDma    = CyTrue;
    status = CyU3PUartSetConfig (&uartConfig, NULL);
    if (status != CY_U3P_SUCCESS)
    {
        return status;
    }

    /* Set the dma for an inifinity transfer */
    status = CyU3PUartTxSetBlockXfer (0xFFFFFFFF);
    if (status != CY_U3P_SUCCESS)
    {
        return status;
    }
   CyU3PDebugPreamble(0);
    /* Start the debug module for printing log messages. */
    status = CyU3PDebugInit (CY_U3P_LPP_SOCKET_UART_CONS, 8);

    return status;
}

#define gpio_w32(b, off)  do { } while (0)
#define gpio_r32(off)   do { } while (0)

#define I2C_STATE_NACK     1
#define I2C_STATE_ACK      0
#define I2C_STATE_IDLE    -1
#define I2C_STATE_START   -2
#define I2C_STATE_STOP    -3
#define I2C_STATE_DEVICE  -4
#define I2C_STATE_TIMEOUT -5

#define I2C_RET_OK     0
#define I2C_RET_END    -1

#define I2C_DEV_OFFS  0x0
#define I2C_DEV_DATA  0x1



static inline int32_t slave_data_receive(struct i2c_slave *slave);
static inline int32_t slave_data_send(struct i2c_slave *slave);
static inline int32_t slave_byte_read(struct i2c_slave *slave);
static inline int32_t slave_byte_write(struct i2c_slave *slave, uint8_t val);
static inline int32_t i2c_dev_address_check(struct i2c_slave *slave, int32_t val);
static inline void i2c_ack_send(struct i2c_slave *slave);
static inline int32_t i2c_ack_read(struct i2c_slave *slave);
static inline void i2c_pins_init(struct i2c_slave *slave);
static inline int32_t i2c_scl_get(struct i2c_slave *slave);
static inline int32_t i2c_sda_get(struct i2c_slave *slave);
static inline void i2c_sda_set(struct i2c_slave *slave, int32_t val);
static inline int32_t wait_for_scl(struct i2c_slave *slave, int32_t level);
//static inline int32_t gpio_value_get(uint32_t gpio);
static inline void gpio_value_set(uint32_t gpio, int32_t val);
static inline void gpio_direction_input(uint32_t gpio);
static inline int32_t i2c_slave_init(void);
static inline int32_t i2c_slave_sda_interrupt_callback(struct i2c_slave *slave);
static inline int32_t i2c_slave_cleanup(struct i2c_slave *slave);
//static inline void gpio_direction_output(uint32_t gpio, int32_t val);
static inline void gpio_direction_output(uint32_t gpio, int val);

static inline void delay(uint32_t count);

//struct i2c_slave *slave;

//#define slave->scl 26;
//#define slave->sda 27;

//slave->scl = 26;
//slave->sda = 27;

int32_t i2c_slave_init()
{
  uint32_t i, j;
 //struct i2c_slave *slave;
  uint8_t slave_addr[I2C_SLAVE_DEV_MAX] =
  {
    /* 7 bits address */
    0xA0>>0x1, 0xB0>>0x1, 0xC0>>0x1, 0xD0>>0x1, 0xE0>>0x1
  };

  //slave = malloc(sizeof(struct i2c_slave));
  slave= CyU3PMemAlloc (0x40);
  slave->scl= 58;
  slave->sda= 59;
  if(!slave)
  {
    return -1;
  }

  for(j = 0; j < I2C_SLAVE_DEV_MAX; j++)
  {
    slave->dev[j].data_offs = 0;
    slave->dev[j].addr = slave_addr[j];
    for(i = 0; i < sizeof(slave->dev[0].data); i++)
    {
      slave->dev[j].data[i] = 0;
    }
  }

  i2c_pins_init(slave);
  return 0;
}

int32_t i2c_slave_cleanup(struct i2c_slave *slave)
{
  free(slave);
  slave = NULL;
  return 0;
}

int32_t i2c_slave_sda_interrupt_callback(struct i2c_slave *slave)
{
  int32_t val;
  uint32_t idx;

  if(wait_for_scl(slave, 0) == I2C_RET_END)
  {
    /* timeout */
  //  goto end;
  }

  slave->state = I2C_STATE_START;

  while(slave->state == I2C_STATE_START)
  {
    /* read address + R/W bit */
    val = slave_byte_read(slave);

    CyU3PDebugPrint (4, "i2c data is %d\n",val);
   if(val == I2C_RET_END)
    {
      if(slave->state == I2C_STATE_START)
      {
        continue;
      }
       //timeout, stop
      goto end;
    }

    idx = i2c_dev_address_check(slave, val);
    if(idx == I2C_RET_END)
    {
      // device address mismatch
      goto end;
    }

    slave->state = I2C_STATE_DEVICE;

    // send ACK
    i2c_ack_send(slave);

    slave->dev_idx = idx;

     //check R/W bit
    if(val & 1)
    {
      //TODO: bug.master can read data without pre-send device data offset
      val = slave_data_send(slave);
      if(val == I2C_RET_END)
      {
        if(slave->state == I2C_STATE_START)
        {
          continue;
        }
        else
        {
          goto end;
        }
      }
    }
    else
    {
      val = slave_data_receive(slave);
      if(val == I2C_RET_END)
      {
        if(slave->state == I2C_STATE_START)
        {
          continue;
        }
        else
        {
          goto end;
        }
      }
    }
  }

  /* wait scl and sda high */
 while (!(i2c_scl_get(slave) && i2c_sda_get(slave)));
  slave->state = I2C_STATE_IDLE;
  return 0;

end:
 // i2c_pins_init(slave);
  slave->state = I2C_STATE_IDLE;
  return -1;
}

static inline int32_t i2c_scl_get(struct i2c_slave *slave)
{
 // int32_t val = gpio_value_get(slave->scl);
 // return val;
	CyU3PReturnStatus_t status;
	status=CyU3PGpioGetValue(58,&highlow);
  return highlow;
}

static inline void i2c_sda_set(struct i2c_slave *slave, int32_t val)
{
  if (val)

    gpio_direction_input(slave->sda);

  else
    gpio_direction_output(slave->sda, 0);
}

static inline int32_t i2c_sda_get(struct i2c_slave *slave)
{
 // int32_t val = gpio_value_get(slave->sda);
  //return val;
  CyU3PGpioGetValue(slave->sda,&highlow);
    return highlow;
}

static inline void i2c_pins_init(struct i2c_slave *slave)
{
  //gpio_direction_input(slave->scl);
	CyFxGpioInit ();
 // gpio_direction_input(slave->sda);
}

static inline void gpio_direction_input(uint32_t gpio)
{
  //(void)gpio;
	CyU3PGpioSimpleConfig_t gpioConfig;
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

	    gpioConfig.outValue = CyFalse;
	    gpioConfig.driveLowEn = CyTrue;
	    gpioConfig.driveHighEn = CyTrue;
	    gpioConfig.inputEn = CyTrue;
	    gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
	    apiRetStatus = CyU3PGpioSetSimpleConfig( slave->sda, &gpioConfig);

}

static inline void gpio_direction_output(uint32_t gpio, int val)
{
  gpio_value_set(gpio, val);
	CyU3PGpioSimpleConfig_t gpioConfig;
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

		    gpioConfig.outValue = CyTrue;
		    gpioConfig.driveLowEn = CyTrue;
		    gpioConfig.driveHighEn = CyTrue;
		    gpioConfig.inputEn = CyFalse;
		    gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
		    apiRetStatus = CyU3PGpioSetSimpleConfig( slave->sda, &gpioConfig);

}

/*static inline int32_t gpio_value_get(uint32_t gpio)
{
  return 1;
}*/

static inline void gpio_value_set(uint32_t gpio, int32_t val)
{
  //(void)gpio;
  //(void)val;
  CyU3PGpioSetValue (slave->sda, CyTrue);
}

static void i2c_slave_store_data(struct i2c_slave *slave, uint8_t flag, uint8_t data)
{
  switch(flag)
  {
    case I2C_DEV_OFFS:
      slave->dev[slave->dev_idx].data_offs = data;
      break;

    case I2C_DEV_DATA:
      slave->dev[slave->dev_idx].data[slave->dev[slave->dev_idx].data_offs] = data;
      slave->dev[slave->dev_idx].data_offs++;
      break;

    default:
      break;
  }
}

static inline int32_t i2c_dev_address_check(struct i2c_slave *slave, int32_t val)
{
  uint32_t i;
  for(i = 0; i < I2C_SLAVE_DEV_MAX; i++)
  {
    if(slave->dev[i].addr == (val>>1))
      return i;
  }
  return -1;
}

static inline void delay(uint32_t count)
{
  while (--count > 0);
}

static inline int32_t wait_for_scl(struct i2c_slave *slave, int32_t level)
{
  uint32_t i = 1000;
  while(i2c_scl_get(slave) != level)
	 // while(i2c_scl_get(slave) != level)
  {
    if(--i == 0)
    {
      slave->state = I2C_STATE_TIMEOUT;
      return I2C_RET_END;
    }
    delay(1);
  }
  return I2C_RET_OK;
}

static inline int32_t slave_byte_write(struct i2c_slave *slave, uint8_t val)
{
  uint8_t i;

  for(i = 0; i < 8; i++)
  {
    i2c_sda_set(slave, val & (1 << (7-i)));
    if(wait_for_scl(slave, 1) == I2C_RET_END)
    {
      return I2C_RET_END;
    }
    if(wait_for_scl(slave, 0) == I2C_RET_END)
    {
      return I2C_RET_END;
    }
  }
  i2c_sda_set(slave, 1);
  return I2C_RET_OK;
}

static inline int32_t slave_byte_read(struct i2c_slave *slave)
{
  uint8_t i;
  int32_t val, temp;

  for(i = 0x0; i < 0x8; i++)
  {
    while(!i2c_scl_get(slave))
    {
      //TODO:timeout check
    }

    val = (val<<0x1) | i2c_sda_get(slave);
    CyU3PDebugPrint (4, "i2c data is %d\n",val);
    while(i2c_scl_get(slave))
    {
      //TODO:timeout check
      /* sda is drivered by master now.
       * if it changes when scl is high,stop or start happened */
      temp = i2c_sda_get(slave);
      if(!i2c_scl_get(slave))
        break;
      if((val & 1) != temp)
      {
        if(temp)
        {
          slave->state = I2C_STATE_STOP;
        }
        else
        {
          slave->state = I2C_STATE_START;
        }
        return I2C_RET_END;
      }
    }
  }
  return I2C_RET_OK;
}

static inline void i2c_ack_send(struct i2c_slave *slave)
{
  /* slave driver sda to low for ACK */
  i2c_sda_set(slave, 0);
  /* wait master read(scl rising edge trigger) ACK */
  while(!i2c_scl_get(slave))
  {
    //TODO:timeout check
  }

  /* wait scl to low */
  while(i2c_scl_get(slave))
  {
    //TODO:timeout check
  }
  /* slave release sda */
  i2c_sda_set(slave, 1);
}

static inline int32_t i2c_ack_read(struct i2c_slave *slave)
{
  int32_t val, temp;

  /* wait master set sda */
  while(!i2c_scl_get(slave))
  {
    //TODO:timeout check
  }
  /* read ACK */
  val = i2c_sda_get(slave);
  /* keep checking sda when scl high */
  while(i2c_scl_get(slave))
  {
    //TODO:timeout check
    /* sda is drivered by master now.
     * if it changes when scl is high,stop or start happened */
    temp = i2c_sda_get(slave);
    if(!i2c_scl_get(slave))
      break;

    if(val != temp)
    {
      if(temp)
      {
        slave->state = I2C_STATE_STOP;
      }
      else
      {
        slave->state = I2C_STATE_START;
      }
      return I2C_RET_END;
    }
  }

  if(val == 0x0)
  {
    /* ACK */
    return I2C_RET_OK;
  }
  else
  {
    /* NACK */
    return I2C_RET_END;
  }
}

static inline int32_t slave_data_send(struct i2c_slave *slave)
{
  uint8_t val, offs;

  offs = slave->dev[slave->dev_idx].data_offs;
  do
  {
    val = slave->dev[slave->dev_idx].data[offs];
    offs ++;
    if (slave_byte_write(slave, val) == I2C_RET_END)
    {
      return I2C_RET_END;
    }
  } while (i2c_ack_read(slave) == I2C_RET_OK);

  slave->dev[slave->dev_idx].data_offs = 0;
  slave->state = I2C_STATE_IDLE;
  return I2C_RET_OK;
}

static inline int32_t slave_data_receive(struct i2c_slave *slave)
{
  int32_t val = 0;
  uint8_t flag = I2C_DEV_OFFS;

  do
  {
    val = slave_byte_read(slave);
    if (val == I2C_RET_END)
    {
      return I2C_RET_END;
    }

    i2c_ack_send(slave);
    i2c_slave_store_data(slave, flag, val);
    flag = I2C_DEV_DATA;
  } while (val != I2C_RET_END);

  return I2C_RET_OK;
}

/* GPIO interrupt callback handler. This is received from
 * the interrupt context. So DebugPrint API is not available
 * from here. Set an event in the event group so that the
 * GPIO thread can print the event information. */

//CyU3PRegisterGpioCallBack(CyU3PGpioIntrCb_t CyFxGpioIntrCb1)
//{


//}

/*void CyFxGpioIntrCb1 (
        uint8_t gpioId
        )
{
    CyBool_t gpioValue;
    CyBool_t gpioValue1;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PReturnStatus_t Status = CY_U3P_SUCCESS;

    // Get the status of the pin
   //  apiRetStatus=CyU3PGpioGetValue(58,&gpioValue);
     Status=CyU3PGpioGetValue(gpioId,&gpioValue1);


}*/

void CyFxGpioIntrCb (
        uint8_t gpioId /* Indicates the pin that triggered the interrupt */
        )
{
    CyBool_t gpioclk;
int k;
    CyBool_t gpiodata;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    uint8_t val=0;
   // apiRetStatus= CyU3PGpioSimpleGetValue(gpioId,&gpiodata);
 //  apiRetStatus=CyU3PGpioGetValue(58,&highlow);
  //apiRetStatus= CyU3PGpioSimplegetValue(gpioId,&highlow);
    /* Get the status of the pin */
   //  apiRetStatus=CyU3PGpioGetValue(58,&gpioValue);
  // if(gpioId==58)
  //  {
   // apiRetStatus=CyU3PGpioSetValue(50,0);
   // for(k=0;k<50000000;k++)
   // {
    val = (*gpio_58_reg & 0x2);

    if(val==0x2)
    {
  *gpio_50_reg = (*gpio_50_reg | 0x01);
    }




   // }
  //  apiRetStatus=CyU3PGpioSetValue(50,1);
     //val=
    // val1 = (val1<<0x1)|gpioclk;
  //  }

  // else
    //    {
       //  Status=CyU3PGpioGetValue(gpioId,&gpiodata);
    //    }

   apiRetStatus = CyU3PGpioGetValue (gpioId, &gpiodata);
   if (apiRetStatus == CY_U3P_SUCCESS)
    {
        //Check status of the pin
      if (gpiodata == CyTrue)
      {
            //Set GPIO high event
    	//  *gpio_50_reg = (*gpio_50_reg | 0x01);
    	  //apiRetStatus=CyU3PGpioSetValue(50,1);
            CyU3PEventSet(&glFxGpioAppEvent, CY_FX_GPIOAPP_GPIO_HIGH_EVENT,
                    CYU3P_EVENT_OR);
      }
       else
       {
    	  //*gpio_50_reg = (*gpio_50_reg | 0x00);
    	  // *gpio_50_reg = (*gpio_50_reg & ~(0x01));
            //Set GPIO low Event
    	  // apiRetStatus=CyU3PGpioSetValue(50,0);
          CyU3PEventSet(&glFxGpioAppEvent, CY_FX_GPIOAPP_GPIO_LOW_EVENT,
                  CYU3P_EVENT_OR);
        }
    }




}

void
CyFxGpioInit (void)
{
    CyU3PGpioClock_t gpioClock;
    CyU3PGpioSimpleConfig_t gpioConfig;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Init the GPIO module */
    gpioClock.fastClkDiv = 2;
    gpioClock.slowClkDiv = 0;
    gpioClock.simpleDiv = CY_U3P_GPIO_SIMPLE_DIV_BY_2;
    gpioClock.clkSrc = CY_U3P_SYS_CLK;
    gpioClock.halfDiv = 0;

    apiRetStatus = CyU3PGpioInit(&gpioClock, CyFxGpioIntrCb);
    if (apiRetStatus != 0)
    {
        /* Error Handling */
        CyU3PDebugPrint (4, "CyU3PGpioInit failed, error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }
    else
    {
    	CyU3PDebugPrint (4, "CyU3PGpioInit succeeded \t", apiRetStatus);
    }

    apiRetStatus = CyU3PDeviceGpioOverride (slave->sda, CyTrue);
       if (apiRetStatus != 0)
       {
           /* Error Handling */
           CyU3PDebugPrint (4, "CyU3PDeviceGpioOverride i2c_data failed, error code = %d\n",
                   apiRetStatus);
           CyFxAppErrorHandler(apiRetStatus);
       }
       else
       {
    	   CyU3PDebugPrint (4, "CyU3PDeviceGpioOverride i2c_data succeeded \t", apiRetStatus);
       }
    /* Configure GPIO 45 as input with interrupt enabled for both edges */
    gpioConfig.outValue = CyFalse;
    gpioConfig.inputEn = CyTrue;
    gpioConfig.driveLowEn = CyFalse;
    gpioConfig.driveHighEn = CyFalse;
    gpioConfig.intrMode = CY_U3P_GPIO_INTR_NEG_EDGE;
    apiRetStatus = CyU3PGpioSetSimpleConfig(slave->sda, &gpioConfig);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        /* Error handling */
        CyU3PDebugPrint (4, "CyU3PGpioSetSimpleConfig i2c_data failed, error code = %d\n",
                apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }
    else
    {
    	 CyU3PDebugPrint (4, "CyU3PGpioSetSimpleConfig i2c_data succeeded \t", apiRetStatus);
    }

 /*  gpioClock.fastClkDiv = 2;
        gpioClock.slowClkDiv = 0;
        gpioClock.simpleDiv = CY_U3P_GPIO_SIMPLE_DIV_BY_2;
        gpioClock.clkSrc = CY_U3P_SYS_CLK;
        gpioClock.halfDiv = 0;

    apiRetStatus = CyU3PGpioInit(&gpioClock, CyFxGpioIntrCb1);
        if (apiRetStatus != 0)
        {
            // Error Handling
            CyU3PDebugPrint (4, "CyU3PGpioInit failed, error code = %d\n", apiRetStatus);
            CyFxAppErrorHandler(apiRetStatus);
        }*/

    /* Override GPIO 21 as this pin is associated with GPIF Control signal.
     * The IO cannot be selected as GPIO by CyU3PDeviceConfigureIOMatrix call
     * as it is part of the GPIF IOs. Override API call must be made with
     * caution as this will change the functionality of the pin. If the IO
     * line is used as part of GPIF and is connected to some external device,
     * then the line will no longer behave as a GPIF IO.. Here CTL4 line is
     * not used and so it is safe to override.  */
    apiRetStatus = CyU3PDeviceGpioOverride (slave->scl, CyTrue);
    if (apiRetStatus != 0)
    {
        /* Error Handling */
        CyU3PDebugPrint (4, "CyU3PDeviceGpioOverride i2c_clcok failed, error code = %d\n",
                apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }
    else
        {
        	 CyU3PDebugPrint (4, "CyU3PDeviceGpioOverride i2c_clcok succeeded\t ", apiRetStatus);
        }

    /* Configure GPIO 21 as output */
    gpioConfig.outValue = CyFalse;
    gpioConfig.driveLowEn = CyFalse;
    gpioConfig.driveHighEn = CyFalse;
    gpioConfig.inputEn = CyTrue;
    gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
    apiRetStatus = CyU3PGpioSetSimpleConfig(slave->scl, &gpioConfig);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        /* Error handling */
        CyU3PDebugPrint (4, "CyU3PGpioSetSimpleConfig i2c_clcok failed, error code = %d\n",
                apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }
    else
            {
            	 CyU3PDebugPrint (4, "CyU3PGpioSetSimpleConfig i2c_clcok succeeded\t", apiRetStatus);
            }

    //led connected gpio
    apiRetStatus = CyU3PDeviceGpioOverride (50, CyTrue);
        if (apiRetStatus != 0)
        {
            // Error Handling
            CyU3PDebugPrint (4, "CyU3PDeviceGpioOverride LED failed, error code = %d\n",
                    apiRetStatus);
            CyFxAppErrorHandler(apiRetStatus);
        }
        else
            {
            	 CyU3PDebugPrint (4, "CyU3PDeviceGpioOverride LED succeeded\t ", apiRetStatus);
            }

        /* Configure GPIO 21 as output */
        gpioConfig.outValue = CyFalse;
        gpioConfig.driveLowEn = CyFalse;
        gpioConfig.driveHighEn = CyTrue;
        gpioConfig.inputEn = CyFalse;
        gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
        apiRetStatus = CyU3PGpioSetSimpleConfig(50, &gpioConfig);
        if (apiRetStatus != CY_U3P_SUCCESS)
        {
            /* Error handling */
            CyU3PDebugPrint (4, "CyU3PGpioSetSimpleConfig LED failed, error code = %d\n",
                    apiRetStatus);
            CyFxAppErrorHandler(apiRetStatus);
        }
        else
                {
                	 CyU3PDebugPrint (4, "CyU3PGpioSetSimpleConfig LED succeeded\t", apiRetStatus);
                }
}

/* Entry function for the gpioOutputThread */
void
GpioOutputThread_Entry (
        uint32_t input)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Initialize Debug module */
    apiRetStatus = CyFxDebugInit();
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        /* Error handling */
        CyU3PDebugPrint (4, "Debug module initialization failed, error code = %d\n",
                apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }
    else
                {
                	 CyU3PDebugPrint (4, "Debug module initialization succeeded\t", apiRetStatus);
                }

    /* Initialize GPIO module. */
    i2c_slave_init();
  // CyFxGpioInit ();


    CyU3PDebugPrint (4, "GPIO OUTPUT THRAD ENTRY \t ",0);
    for (;;)

    {
    	//*gpio_50_reg = (*gpio_50_reg & ~(0x01));
    	// apiRetStatus=CyU3PGpioSetValue(50,1);
    	//CyU3PDebugLog(3, 0xff, status);
    	//i2c_slave_sda_interrupt_callback(&slave);
    	//apiRetStatus= CyU3PGpioSimpleGetValue(58,&highlow);
    	//apiRetStatus=CyU3PGpioGetValue(59,&highlow1);

    	//i2c_slave_sda_interrupt_callback(&slave);

    	//apiRetStatus= CyU3PGpioSimpleGetValue(58,&highlow);
        /* Wait for two seconds */
       // CyU3PThreadSleep(100);
       // apiRetStatus=CyU3PGpioGetValue(58,&highlow2);


    }
}

/* Entry function for the gpioInputThread */
void
GpioInputThread_Entry (
        uint32_t input)
{
    uint32_t eventFlag;
    CyU3PReturnStatus_t txApiRetStatus = CY_U3P_SUCCESS;
    struct i2c_slave *slave1;
   // *gpio_50_reg = (*gpio_50_reg | 0x01);
    CyU3PDebugPrint (4, "GPIO INPUT THREAD ENTRY FUNCTION 1 \t");

    for (;;)
    {
    	//*gpio_50_reg = (*gpio_50_reg | 0x01);
    	//CyU3PDebugPrint (4, "GPIO INPUT THREAD ENTRY FUNCTION 2 \t");
        /* Wait for a GPIO event */
        txApiRetStatus = CyU3PEventGet (&glFxGpioAppEvent,
                (CY_FX_GPIOAPP_GPIO_HIGH_EVENT | CY_FX_GPIOAPP_GPIO_LOW_EVENT),
                CYU3P_EVENT_OR_CLEAR, &eventFlag, CYU3P_WAIT_FOREVER);

        if (txApiRetStatus == CY_U3P_SUCCESS)
        {

        	//txApiRetStatus=CyU3PGpioGetValue(58,&highlow);

            if (eventFlag & CY_FX_GPIOAPP_GPIO_HIGH_EVENT)
            {
            	// *gpio_50_reg = (*gpio_50_reg | 0x01);
            	//eventFlag=0;
                /* Print the status of the pin */
            	//i2c_slave_sda_interrupt_callback(&slave);

                CyU3PDebugPrint (4, "i2c data is set to high\n");
            }
            else
            {
                /* Print the status of the pin */
                CyU3PDebugPrint (4, "GPIO 45 is set to low\n");
            }
        }
    }
}

/* Application define function which creates the threads. */
void
CyFxApplicationDefine (
        void)
{
    void *ptr = NULL;
    uint32_t retThrdCreate = CY_U3P_SUCCESS;

    /* Allocate the memory for the threads */
    ptr = CyU3PMemAlloc (CY_FX_GPIOAPP_THREAD_STACK);

    /* Create the thread for the application */
    retThrdCreate = CyU3PThreadCreate (&gpioOutputThread,        /* GPIO Example App Thread structure */
                          "21:GPIO_simple_output",               /* Thread ID and Thread name */
                          GpioOutputThread_Entry,                /* GPIO Example App Thread Entry function */
                          0,                                     /* No input parameter to thread */
                          ptr,                                   /* Pointer to the allocated thread stack */
                          CY_FX_GPIOAPP_THREAD_STACK,            /* Thread stack size */
                          CY_FX_GPIOAPP_THREAD_PRIORITY,         /* Thread priority */
                          CY_FX_GPIOAPP_THREAD_PRIORITY,         /* Pre-emption threshold for the thread. */
                          CYU3P_NO_TIME_SLICE,                   /* No time slice for the application thread */
                          CYU3P_AUTO_START                       /* Start the Thread immediately */
                          );

    /* Check the return code */
    if (retThrdCreate != 0)
    {
        /* Thread creation failed with the error code retThrdCreate */

        /* Add custom recovery or debug actions here */

        /* Application cannot continue */
        /* Loop indefinitely */
        while(1);
    }

    /* Allocate the memory for the threads */
    ptr = CyU3PMemAlloc (CY_FX_GPIOAPP_THREAD_STACK);

    /* Create the thread for the application */
    retThrdCreate = CyU3PThreadCreate (&gpioInputThread,          /* GPIO Example App Thread structure */
                          "22:GPIO_simple_input",                 /* Thread ID and Thread name */
                          GpioInputThread_Entry,                  /* GPIO Example App Thread entry function */
                          0,                                      /* No input parameter to thread */
                          ptr,                                    /* Pointer to the allocated thread stack */
                          CY_FX_GPIOAPP_THREAD_STACK,             /* Thread stack size */
                          CY_FX_GPIOAPP_THREAD_PRIORITY,          /* Thread priority */
                          CY_FX_GPIOAPP_THREAD_PRIORITY,          /* Pre-emption threshold for the thread */
                          CYU3P_NO_TIME_SLICE,                    /* No time slice for the application thread */
                          CYU3P_AUTO_START                        /* Start the Thread immediately */
                          );

    /* Check the return code */
    if (retThrdCreate != 0)
    {
        /* Thread creation failed with the error code retThrdCreate */

        /* Add custom recovery or debug actions here */

        /* Application cannot continue */
        /* Loop indefinitely */
        while(1);
    }

    /* Create GPIO application event group */
    retThrdCreate = CyU3PEventCreate(&glFxGpioAppEvent);
    if (retThrdCreate != 0)
    {
        /* Event group creation failed with the error code retThrdCreate */

        /* Add custom recovery or debug actions here */

        /* Application cannot continue */
        /* Loop indefinitely */
        while(1);
    }
}

/*
 * Main function
 */
int
main (void)
{
    CyU3PIoMatrixConfig_t io_cfg;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    /* Initialize the device */

    CyU3PSysClockConfig_t clkCfg;

        /* setSysClk400 clock configurations */
        clkCfg.setSysClk400 = CyTrue;   /* FX3 device's master clock is set to a frequency > 400 MHz */
        clkCfg.cpuClkDiv = 2;           /* CPU clock divider */
        clkCfg.dmaClkDiv=2;
        clkCfg.mmioClkDiv = 2;          /* MMIO clock divider */
        clkCfg.useStandbyClk = CyFalse; /* device has no 32KHz clock supplied */
        clkCfg.clkSrc = CY_U3P_SYS_CLK; /* Clock source for a peripheral block  */

    status = CyU3PDeviceInit (&clkCfg);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Initialize the caches. Enable both Instruction and Data Caches. */
    status = CyU3PDeviceCacheControl (CyTrue, CyTrue, CyTrue);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Configure the IO matrix for the device. On the FX3 DVK board,
     * the COM port is connected to the IO(53:56). This means that
     * either DQ32 mode should be selected or lppMode should be set
     * to UART_ONLY. Here we are choosing UART_ONLY configuration. */
    CyU3PMemSet ((uint8_t *)&io_cfg, 0, sizeof(io_cfg));
    io_cfg.isDQ32Bit = CyFalse;
    io_cfg.s0Mode = CY_U3P_SPORT_INACTIVE;
    io_cfg.s1Mode = CY_U3P_SPORT_INACTIVE;
    io_cfg.useUart   = CyTrue;
    io_cfg.useI2C    = CyFalse;
    io_cfg.useI2S    = CyFalse;
    io_cfg.useSpi    = CyFalse;
    io_cfg.lppMode   = CY_U3P_IO_MATRIX_LPP_DEFAULT;
    /* GPIO 45 is used as input pin. GPIO 21 is also used but cannot
     * be selected here as it is part of the GPIF IOs (CTL4). Since
     * this IO is not used, it can be overridden to become a GPIO by
     * invoking the CyU3PDeviceGpioOverride call. */
    io_cfg.gpioSimpleEn[0]  = 0;
   // io_cfg.gpioSimpleEn[1]  = 0x0030000; /* GPIO 45 */
    io_cfg.gpioSimpleEn[1]  = 0;
    io_cfg.gpioComplexEn[0] = 0;
    io_cfg.gpioComplexEn[1] = 0;
    status = CyU3PDeviceConfigureIOMatrix (&io_cfg);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* This is a non returnable call for initializing the RTOS kernel */
    CyU3PKernelEntry ();

    /* Dummy return to make the compiler happy */
    return 0;

handle_fatal_error:
    /* Cannot recover from this error. */
    while (1);
}

/* [ ] */

